@@ -14,7 +14,7 @@ typedef struct _IBusM17NEngine IBusM17NEngine;
 typedef struct _IBusM17NEngineClass IBusM17NEngineClass;
 
 struct _IBusM17NEngine {
-    IBusEngine parent;
+    IBusEngineSimple parent;
 
     /* members */
     MInputContext *context;
@@ -30,7 +30,7 @@ struct _IBusM17NEngine {
 };
 
 struct _IBusM17NEngineClass {
-    IBusEngineClass parent;
+    IBusEngineSimpleClass parent;
 
     /* configurations are per class */
     GSettings *gsettings;
@@ -94,7 +94,7 @@ static void ibus_m17n_engine_update_preedit (IBusM17NEngine *m17n);
 static void ibus_m17n_engine_update_lookup_table
                                             (IBusM17NEngine *m17n);
 
-static IBusEngineClass *parent_class = NULL;
+static IBusEngineSimpleClass *parent_class = NULL;
 
 void
 ibus_m17n_init (IBusBus *bus)
@@ -160,6 +160,7 @@ ibus_m17n_engine_get_type_for_name (const gchar *engine_name)
 {
     GType type;
     gchar *type_name, *lang = NULL, *name = NULL;
+    int i;
 
     GTypeInfo type_info = {
         sizeof (IBusM17NEngineClass),
@@ -178,6 +179,12 @@ ibus_m17n_engine_get_type_for_name (const gchar *engine_name)
         g_free (name);
         return G_TYPE_INVALID;
     }
+    for (i = 0; lang[i] != '\0'; i++) {
+      lang[i] = g_ascii_tolower (lang[i]);
+    }
+    for (i = 0; name[i] != '\0'; i++) {
+      name[i] = g_ascii_tolower (name[i]);
+    }
     lang[0] = g_ascii_toupper (lang[0]);
     name[0] = g_ascii_toupper (name[0]);
     type_name = g_strdup_printf ("IBusM17N%s%sEngine", lang, name);
@@ -185,10 +192,10 @@ ibus_m17n_engine_get_type_for_name (const gchar *engine_name)
     g_free (name);
 
     type = g_type_from_name (type_name);
-    g_assert (type == 0 || g_type_is_a (type, IBUS_TYPE_ENGINE));
+    g_assert (type == 0 || g_type_is_a (type, IBUS_TYPE_ENGINE_SIMPLE));
 
     if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_ENGINE,
+        type = g_type_register_static (IBUS_TYPE_ENGINE_SIMPLE,
                                        type_name,
                                        &type_info,
                                        (GTypeFlags) 0);
@@ -208,7 +215,7 @@ ibus_m17n_engine_class_init (IBusM17NEngineClass *klass)
     IBusM17NEngineConfig *engine_config;
 
     if (parent_class == NULL)
-        parent_class = (IBusEngineClass *) g_type_class_peek_parent (klass);
+        parent_class = (IBusEngineSimpleClass *) g_type_class_peek_parent (klass);
 
     object_class->constructor = ibus_m17n_engine_constructor;
     ibus_object_class->destroy = (IBusObjectDestroyFunc) ibus_m17n_engine_destroy;
@@ -238,24 +245,46 @@ ibus_m17n_engine_class_init (IBusM17NEngineClass *klass)
         g_free (name);
         return;
     }
+    MPlist *l = minput_get_title_icon (msymbol (lang), msymbol (name));
+    if (l == NULL) {
+      /*
+	If finding the icon did not work, try it in all upper case.
+	This is a silly hack to make it work with /usr/share/sa-iast.mim which
+	contains (input-method sa IAST ) and has the icon: /usr/share/m17n/icons/sa-IAST.png
+	Without this hack, the gsettings for sa-IAST do not work either.
+	See also: https://github.com/ibus/ibus-m17n/issues/52
+      */
+      int i;
+      gchar *name_uppercase;
+      name_uppercase = g_strdup (name);
+      for (i = 0; name_uppercase[i] != '\0'; i++) {
+	name_uppercase[i] = g_ascii_toupper(name_uppercase[i]);
+      }
+      l = minput_get_title_icon (msymbol (lang), msymbol (name_uppercase));
+      if (l) {
+	g_free(name);
+	name = g_strdup (name_uppercase);
+      }
+      g_free (name_uppercase);
+    }
+    if (l && mplist_key (l) == Mtext) {
+        klass->title = ibus_m17n_mtext_to_utf8 (mplist_value (l));
+        MPlist *n = mplist_next (l);
+        if (n && mplist_key (n) == Mtext) {
+            klass->icon = ibus_m17n_mtext_to_utf8 (mplist_value (n));
+        }
+        else {
+            klass->icon = NULL;
+        }
+    }
+    else {
+        klass->title = NULL;
+        klass->icon = NULL;
+    }
     klass->gsettings = g_settings_new_with_path (
         "org.freedesktop.ibus.engine.m17n",
         g_strdup_printf ("/org/freedesktop/ibus/engine/m17n/%s/%s/",
                          lang, name));
-    MPlist *l = minput_get_title_icon (msymbol (lang), msymbol (name));
-    if (l && mplist_key (l) == Mtext) {
-        klass->title = ibus_m17n_mtext_to_utf8 (mplist_value (l));
-    }
-    else {
-        klass->title = NULL;
-    }
-    MPlist *n = mplist_next (l);
-    if (n && mplist_key (n) == Mtext) {
-        klass->icon = ibus_m17n_mtext_to_utf8 (mplist_value (n));
-    }
-    else {
-        klass->icon = NULL;
-    }
     engine_name = g_strdup_printf ("m17n:%s:%s", lang, name);
     klass->engine_name = g_strdup (engine_name);
     klass->lang = g_strdup (lang);
@@ -380,9 +409,17 @@ ibus_m17n_engine_init (IBusM17NEngine *m17n)
                                            klass->icon,
                                            ibus_text_new_from_string (klass->engine_name),
                                            TRUE,
-                                           FALSE,
-                                           0,
+                                           TRUE,
+                                           PROP_STATE_UNCHECKED,
                                            NULL);
+    /*
+      If a text instead of an icon should be shown at the status property
+      a symbol needs to be set
+    */
+    /*
+    ibus_property_set_symbol(m17n->status_prop,
+			     ibus_text_new_from_string (klass->engine_name));
+    */
     g_object_ref_sink (m17n->status_prop);
     ibus_prop_list_append (m17n->prop_list,  m17n->status_prop);
 
@@ -695,6 +732,21 @@ ibus_m17n_engine_process_key_event (IBusEngine     *engine,
         break;
     }
 
+    if (IBUS_ENGINE_CLASS (parent_class)->process_key_event (engine, keyval, keycode, modifiers)) {
+      if (mtext_len (m17n->context->preedit) > 0) {
+	gchar *buf;
+	buf = ibus_m17n_mtext_to_utf8 (m17n->context->preedit);
+	if (buf) {
+	  IBusText *text;
+	  text = ibus_text_new_from_string (buf);
+	  ibus_engine_commit_text (engine, text);
+	  g_free (buf);
+	}
+	minput_reset_ic (m17n->context);
+      }
+      return TRUE;
+    }
+
     if (modifiers & IBUS_RELEASE_MASK)
         return FALSE;
 
@@ -733,7 +785,7 @@ ibus_m17n_engine_focus_in (IBusEngine *engine)
     ibus_engine_register_properties (engine, m17n->prop_list);
     ibus_m17n_engine_process_key (m17n, Minput_focus_in);
 
-    parent_class->focus_in (engine);
+    IBUS_ENGINE_CLASS (parent_class)->focus_in (engine);
 }
 
 static void
@@ -746,7 +798,7 @@ ibus_m17n_engine_focus_out (IBusEngine *engine)
        m17n-lib. */
     minput_reset_ic (m17n->context);
 
-    parent_class->focus_out (engine);
+    IBUS_ENGINE_CLASS (parent_class)->focus_out (engine);
 }
 
 static void
@@ -754,7 +806,7 @@ ibus_m17n_engine_reset (IBusEngine *engine)
 {
     IBusM17NEngine *m17n = (IBusM17NEngine *) engine;
 
-    parent_class->reset (engine);
+    IBUS_ENGINE_CLASS (parent_class)->reset (engine);
 
     minput_reset_ic (m17n->context);
 }
@@ -762,7 +814,7 @@ ibus_m17n_engine_reset (IBusEngine *engine)
 static void
 ibus_m17n_engine_enable (IBusEngine *engine)
 {
-    parent_class->enable (engine);
+    IBUS_ENGINE_CLASS (parent_class)->enable (engine);
 
     /* Issue a dummy ibus_engine_get_surrounding_text() call to tell
        input context that we will use surrounding-text. */
@@ -773,7 +825,7 @@ static void
 ibus_m17n_engine_disable (IBusEngine *engine)
 {
     ibus_m17n_engine_focus_out (engine);
-    parent_class->disable (engine);
+    IBUS_ENGINE_CLASS (parent_class)->disable (engine);
 }
 
 static void
@@ -782,7 +834,7 @@ ibus_m17n_engine_page_up (IBusEngine *engine)
     IBusM17NEngine *m17n = (IBusM17NEngine *) engine;
 
     ibus_m17n_engine_process_key (m17n, msymbol ("Up"));
-    parent_class->page_up (engine);
+    IBUS_ENGINE_CLASS (parent_class)->page_up (engine);
 }
 
 static void
@@ -792,7 +844,7 @@ ibus_m17n_engine_page_down (IBusEngine *engine)
     IBusM17NEngine *m17n = (IBusM17NEngine *) engine;
 
     ibus_m17n_engine_process_key (m17n, msymbol ("Down"));
-    parent_class->page_down (engine);
+    IBUS_ENGINE_CLASS (parent_class)->page_down (engine);
 }
 
 static void
@@ -802,7 +854,7 @@ ibus_m17n_engine_cursor_up (IBusEngine *engine)
     IBusM17NEngine *m17n = (IBusM17NEngine *) engine;
 
     ibus_m17n_engine_process_key (m17n, msymbol ("Left"));
-    parent_class->cursor_up (engine);
+    IBUS_ENGINE_CLASS (parent_class)->cursor_up (engine);
 }
 
 static void
@@ -812,7 +864,7 @@ ibus_m17n_engine_cursor_down (IBusEngine *engine)
     IBusM17NEngine *m17n = (IBusM17NEngine *) engine;
 
     ibus_m17n_engine_process_key (m17n, msymbol ("Right"));
-    parent_class->cursor_down (engine);
+    IBUS_ENGINE_CLASS (parent_class)->cursor_down (engine);
 }
 
 static void
@@ -836,7 +888,7 @@ ibus_m17n_engine_property_activate (IBusEngine  *engine,
     }
 #endif  /* HAVE_SETUP */
 
-    parent_class->property_activate (engine, prop_name, prop_state);
+    IBUS_ENGINE_CLASS (parent_class)->property_activate (engine, prop_name, prop_state);
 }
 
 static void
